package eu.beesoft.gantt.demo;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.table.AbstractTableModel;
import eu.beesoft.gaia.swing.JDateField;
import eu.beesoft.gaia.swing.SimpleSwingForm;
import eu.beesoft.gantt.Dependency;
import eu.beesoft.gantt.DependencyType;
import eu.beesoft.gantt.JGantt;
import eu.beesoft.gantt.undo.UndoStep;

@SuppressWarnings ("serial")
public class TaskForm extends SimpleSwingForm {

	private JGantt gantt;

	private Task task;

	private JTextField name;

	private JLabel id;

	private JDateField startDate;

	private JDateField endDate;

	private JTextField effort;

	private JCheckBox milestone;

	private JComboBox assignee;

	private JTextArea description;

	private JTable dependencies;

	private DependencyModel dependenciesModel;

	private List<Dependency> myDependencies;

	private Map<DependencyType, String> textByDependencyType = new HashMap<DependencyType, String> ();



	// ------------------------------------------------------ constructor
	/**
	 * 
	 */
	public TaskForm (Task task, JGantt gantt) {
		this.task = task;
		this.gantt = gantt;

		// prepare dependency type names
		textByDependencyType.put (DependencyType.FINISH_TO_START, "Finish-To-Start");
		textByDependencyType.put (DependencyType.START_TO_FINISH, "Start-To-Finish");
		textByDependencyType.put (DependencyType.FINISH_TO_FINISH, "Finish-To-Finish");
		textByDependencyType.put (DependencyType.START_TO_START, "Start-To-Start");

		// task name
		name = createTextField ();
		String s = task.getName ();
		if (s == null) {
			s = "";
		}
		name.setText (s);
		addLabeledComponent ("Name", name);

		// task ID
		id = new JLabel (task.getTaskId ());
		addLabeledComponent ("Task ID", id);

		// description
		description = createTextArea (6);
		s = task.getDescription ();
		if (s != null) {
			description.setText (s);
		}
		addLabeledComponent ("Description", new JScrollPane (description));

		// start date
		startDate = new JDateField ();
		startDate.setDate (task.getStartDate ());
		addLabeledComponent ("Start date", startDate);

		// end date
		endDate = new JDateField ();
		endDate.setDate (task.getEndDate ());
		addLabeledComponent ("End date", endDate);

		// effort
		effort = createTextField (10);
		effort.setText (Long.toString (task.getEffort ()));
		addLabeledComponent ("Effort (mandays)", effort);

		// milestone
		milestone = new JCheckBox ();
		milestone.setSelected (task.isMilestone ());
		addLabeledComponent ("Milestone", milestone);

		// assignee
		assignee = new JComboBox ();
		Project project = task.getProject ();
		if (project != null) {
			List<Resource> resources = project.getResources ();
			assignee.addItem ("");
			if (!resources.isEmpty ()) {
				List<Resource> lr = new ArrayList<Resource> (resources);
				Collections.sort (lr);
				for (Resource r : lr) {
					assignee.addItem (r);
				}
			}
			Resource sel = task.getAssignee ();
			if (sel != null) {
				assignee.setSelectedItem (sel);
			}
		}
		addLabeledComponent ("Assignee", assignee);

		// dependencies
		myDependencies = new ArrayList<Dependency> ();
		for (Dependency dep : task.getDependencies ()) {
			if (dep.getSuccessor () == task) {
				myDependencies.add (dep);
			}
		}
		dependenciesModel = new DependencyModel ();
		dependencies = new JTable (dependenciesModel);

		addLabeledComponent ("Depends on", new JScrollPane (dependencies));

		// filler at the end of form
		addFiller ();
	}



	// ---------------------------------------------------- getActions ()
	/**
	 * 
	 */
	public Action[] getActions () {
		Action[] actions = new Action[5];
		actions[0] = new AddDependencyAction ();
		actions[1] = new EditDependencyAction ();
		actions[2] = new RemoveDependencyAction ();
		actions[3] = new OkAction ();
		actions[4] = new CancelAction ();
		return actions;
	}



	// ------------------------------------------------------------------
	// Inner class D e p e n d e n c y M o d e l
	// ------------------------------------------------------------------
	private class DependencyModel extends AbstractTableModel {

		public int getColumnCount () {
			return 4;
		}



		@Override
		public String getColumnName (int column) {
			switch (column) {
				case 0:
					return "ID";
				case 1:
					return "Task name";
				case 2:
					return "Type";
				default:
					return "Delay";
			}
		}



		public int getRowCount () {
			return myDependencies.size ();
		}



		public Object getValueAt (int row, int column) {
			TaskDependency dep = (TaskDependency) myDependencies.get (row);
			Task dependsOn = (Task) dep.getPredecessor ();
			switch (column) {
				case 0:
					return dependsOn.getTaskId ();
				case 1:
					return dependsOn.getName ();
				case 2:
					DependencyType type = dep.getType ();
					return textByDependencyType.get (type);
				default:
					return dep.getDelay ();
			}
		}
	}



	// ------------------------------------------------------------------
	// Inner class O k A c t i o n
	// ------------------------------------------------------------------
	private class OkAction extends AbstractAction {

		public OkAction () {
			putValue (Action.NAME, "OK");
		}



		public void actionPerformed (ActionEvent event) {
			UndoStep undo = new UndoStep ();
			undo.registerObject (task);
			task.setName (name.getText ());
			task.setDescription (description.getText ());
			task.setStartDate (startDate.getDate ());
			task.setEndDate (endDate.getDate ());
			task.setMilestone (milestone.isSelected ());
			String s = effort.getText ().trim ();
			long e = 0;
			if (!"".equals (s)) {
				try {
					e = Long.parseLong (s);
				}
				catch (NumberFormatException nfe) {
					// empty
				}
			}
			task.setEffort (e);
			Object selAssignee = assignee.getSelectedItem ();
			if (selAssignee instanceof Resource) {
				task.setAssignee ((Resource) selAssignee);
			}
			else {
				task.setAssignee (null);
			}
			undo.end ();
			gantt.getUndoManager ().addEdit (undo);
			destroyDialog ();
		}
	}



	// ------------------------------------------------------------------
	// Inner class C a n c e l A c t i o n
	// ------------------------------------------------------------------
	private class CancelAction extends AbstractAction {

		public CancelAction () {
			putValue (Action.NAME, "Cancel");
		}



		public void actionPerformed (ActionEvent event) {
			destroyDialog ();
		}
	}



	// ------------------------------------------------------------------
	// Inner class A d d D e p e n d e n c y A c t i o n
	// ------------------------------------------------------------------
	private class AddDependencyAction extends AbstractAction {

		public AddDependencyAction () {
			putValue (Action.NAME, "Add dependency");
		}



		public void actionPerformed (ActionEvent event) {
			TaskDependency dep = new TaskDependency ();
			dep.setSuccessor (task);
			dep.setType (DependencyType.FINISH_TO_START);
			DependencyForm form = new DependencyForm (dep);
			form.showDialog ("Add new dependency", new DependencyFormOk (form, true),
			        new DependencyFormCancel (form));
		}
	}



	// ------------------------------------------------------------------
	// Inner class E d i t D e p e n d e n c y A c t i o n
	// ------------------------------------------------------------------
	private class EditDependencyAction extends AbstractAction {

		public EditDependencyAction () {
			putValue (Action.NAME, "Edit");
		}



		public void actionPerformed (ActionEvent event) {
			int row = dependencies.getSelectedRow ();
			if ((row >= 0) && (row < myDependencies.size ())) {
				TaskDependency dep = (TaskDependency) myDependencies.get (row);
				DependencyForm form = new DependencyForm (dep);
				form.showDialog ("Edit dependency", new DependencyFormOk (form, false),
				        new DependencyFormCancel (form));
			}
		}
	}



	// ------------------------------------------------------------------
	// Inner class R e m o v e D e p e n d e n c y A c t i o n
	// ------------------------------------------------------------------
	private class RemoveDependencyAction extends AbstractAction {

		public RemoveDependencyAction () {
			putValue (Action.NAME, "Remove dependency");
		}



		public void actionPerformed (ActionEvent event) {
			int row = dependencies.getSelectedRow ();
			if ((row >= 0) && (row < myDependencies.size ())) {
				myDependencies.remove (row);
				dependenciesModel.fireTableRowsDeleted (row, row);
			}
		}
	}



	// ------------------------------------------------------------------
	// Inner class D e p e n d e n c y F o r m
	// ------------------------------------------------------------------
	private class DependencyForm extends SimpleSwingForm {

		private TaskDependency dependency;

		private JComboBox dependencyDependsOn;

		private JComboBox dependencyType;

		private JTextField dependencyDelay;



		public DependencyForm (TaskDependency dependency) {
			this.dependency = dependency;

			// dependant
			Task dependantTask = (Task) dependency.getSuccessor ();
			JLabel dependant = new JLabel (dependantTask.getName ());
			addLabeledComponent ("Task", dependant);

			// depends on
			Vector<Task> tasks = new Vector<Task> ();
			Project project = dependantTask.getProject ();
			for (Task t : project.getSubtask ()) {
				flatTasks (t, tasks, dependantTask);
			}
			dependencyDependsOn = new JComboBox (tasks);
			Task selected = (Task) dependency.getPredecessor ();
			if (selected != null) {
				dependencyDependsOn.setSelectedItem (selected);
			}
			addLabeledComponent ("Depends on", dependencyDependsOn);

			// type
			dependencyType = new JComboBox ();
			dependencyType.addItem (textByDependencyType.get (DependencyType.FINISH_TO_START));
			dependencyType.addItem (textByDependencyType.get (DependencyType.FINISH_TO_FINISH));
			dependencyType.addItem (textByDependencyType.get (DependencyType.START_TO_START));
			dependencyType.addItem (textByDependencyType.get (DependencyType.START_TO_FINISH));
			DependencyType type = dependency.getType ();
			dependencyType.setSelectedItem (textByDependencyType.get (type));
			addLabeledComponent ("Type", dependencyType);

			// delay
			dependencyDelay = new JTextField ();
			dependencyDelay.setText (Long.toString (dependency.getDelay ()));
			addLabeledComponent ("Delay", dependencyDelay);

			// filler
			addFiller ();
		}



		private void flatTasks (Task task, Vector<Task> result, Task ignore) {
			if (task == ignore) {
				return;
			}
			result.add (task);
			for (Task t : task.getSubtask ()) {
				flatTasks (t, result, ignore);
			}
		}
	}



	// ------------------------------------------------------------------
	// Inner class D e p e n d e n c y F o r m O k
	// ------------------------------------------------------------------
	private class DependencyFormOk extends AbstractAction {

		private DependencyForm form;

		private boolean isNew;



		public DependencyFormOk (DependencyForm form, boolean isNew) {
			this.form = form;
			this.isNew = isNew;
			putValue (Action.NAME, "OK");
		}



		public void actionPerformed (ActionEvent event) {
			Task dependsOn = (Task) form.dependencyDependsOn.getSelectedItem ();
			form.dependency.setPredecessor (dependsOn);
			String type = (String) form.dependencyType.getSelectedItem ();
			for (DependencyType dType : textByDependencyType.keySet ()) {
				String tText = textByDependencyType.get (dType);
				if (tText.equals (type)) {
					form.dependency.setType (dType);
					break;
				}
			}
			long delay = 0;
			String s = form.dependencyDelay.getText ().trim ();
			if (!"".equals (s)) {
				try {
					delay = Long.parseLong (s);
				}
				catch (NumberFormatException nfe) {
					// process user error
				}
			}
			form.dependency.setDelay (delay);
			if (isNew) {
				myDependencies.add (form.dependency);
				dependenciesModel.fireTableRowsInserted (myDependencies.size () - 1, myDependencies
				        .size ());
			}
			else {
				int row = myDependencies.indexOf (form.dependency);
				dependenciesModel.fireTableRowsUpdated (row, row);
			}
			form.destroyDialog ();
		}
	}



	// ------------------------------------------------------------------
	// Inner class D e p e n d e n c y F o r m C a n c e l
	// ------------------------------------------------------------------
	private class DependencyFormCancel extends AbstractAction {

		private DependencyForm form;



		public DependencyFormCancel (DependencyForm form) {
			this.form = form;
			putValue (Action.NAME, "Cancel");
		}



		public void actionPerformed (ActionEvent event) {
			form.destroyDialog ();
		}
	}
}
